#!/usr/bin/env scheme-chicken

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ---START CHICKEN PRELUDE---

(require 'srfi-1)

(define-syntax get-optional
  (syntax-rules ()
    ((get-optional opt default)
     (let ((tmp opt))
       (if (pair? tmp) (car tmp) default)))))

(define-syntax inc!
  (syntax-rules ()
    ((inc! var) (inc! var 1))
    ((inc! var val) (set! var (fx+ var val)))))

(define-syntax push!
  (syntax-rules ()
    ((push! ls x) (set! ls (cons x ls)))))

(define-syntax dotimes
  (syntax-rules ()
    ((dotimes (var limit) body ...)
     (let ((l limit))
       (do ((var 0 (fx+ var 1)))
           ((fx= var l))
         body ...)))))

(define (bit-set? index a)
  (odd? (arithmetic-shift a (- index))))

(define string-join string-intersperse)

(define (warn fmt . args)
  (apply format (current-error-port) args)
  (newline (current-error-port)))

(define (quotient&remainder a b)
  (values (quotient a b) (remainder a b)))

(define (port->list reader port)
  (let loop ((ls '()))
    (let ((x (reader port)))
      (if (eof-object? x)
        (reverse ls)
        (loop (cons x ls))))))

(define keyword? symbol?)

(define hash-table-get hash-table-ref)
(define hash-table-put! hash-table-set!)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; need gmp to emulate the test rng

; (require 'gmp)

; (define (make-rng init)
;   (define (shift n) (mpz_mul_ui n n 22695477) (mpz_add_ui n n 1))
;   (let ((s0 (make-mpz_t))
;         (x0 (make-mpz_t)))
;     (mpz_init s0)
;     (mpz_init x0)
;     (mpz_set_ui s0 init)
;     (shift s0) (shift s0) (shift s0) (shift s0)
;     (lambda (n)
;       (mpz_tdiv_q_ui x0 s0 65536)
;       (mpz_tdiv_r_ui x0 x0 16384)
;       (shift s0)
;       (mpz_tdiv_r_ui x0 x0 n))))

; (define (make-rng init)
;   (define (shift n) (fx+ 1 (fx* n 22695477)))
;   (let ((s0 (shift (shift (shift (shift init))))))
;     (lambda (n)
;       (let ((x0 (fxmod (fx/ s0 65536) 16384)))
;         (set! s0 (shift s0))
;         (fxmod x0 n)))))

;; if it's not identical we're better off with the faster builtin random

(define (make-rng init)
  random)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
